const request = require('request');
const { Readable } = require('stream');
const fs = require("fs");
const unzip = require('unzip-stream');
const nodePath = require('path');

class SangjogZipExtractor {

    static extractFromBuffArr(buffArr, options, completeFn) {
        const buff = Buffer.concat(buffArr);
        const stream = Readable.from(buff).pipe(unzip.Parse());
        SangjogZipExtractor.extractFromStream(stream, options, completeFn);
    }

    static extractFromStream(stream, options, completeFn) {
        stream.on('entry', (entry, e, cb) => {
            const { path, isDirectory } = entry;
            const { inWhichDir, zipFolderName, devProjectName } = options;
            const projPath = nodePath.join(inWhichDir, path);
            const sangjogDirPath = projPath.replace(zipFolderName, devProjectName);
            if(path) {
                if(isDirectory) {
                    if(!fs.existsSync(sangjogDirPath)) {
                        fs.mkdirSync(sangjogDirPath);
                    }
                } else {
                    entry.pipe(fs.createWriteStream(sangjogDirPath));
                }
            } else {
                entry.autodrain();
            }
        });
        completeFn();
    }

    static downloadFile(filePath, options, completeFn) {
        const buffArr = [];
        request.get(filePath).on('data', (buff)=>{
            buffArr.push(buff);
        }).on('error', (err) => {
            if(err && err.host && err.host.indexOf('github.com')>-1) {
                console.log('Network issue. trying again....');
                SangjogZipExtractor.downloadFile(filePath, options, completeFn);
            } else {
                console.error(err);
            }
        }).on('complete', (response)=>{
            if(response && response.statusCode===200) {
                SangjogZipExtractor.extractFromBuffArr(buffArr, options, completeFn);
            }
        });
    }

}

module.exports = SangjogZipExtractor;